@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -47,8 +47,8 @@ extern int option_quickest;
 
 /* Local functions */
 
-static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,index_t finish_node,index_t finish_segment);
-static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,index_t start_node,index_t finish_node);
+static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t finish_node,index_t finish_segment);
+static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t start_node,index_t finish_node);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -170,7 +170,15 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
        /* must obey one-way restrictions (unless profile allows) */
        if(profile->oneway && IsOnewayTo(segmentp,node1))
-          goto endloop;
+         {
+          if(profile->allow!=Transports_Bicycle)
+             goto endloop;
+
+          wayp=LookupWay(ways,segmentp->way,1);
+
+          if(!(wayp->type&Highway_CycleBothWays))
+             goto endloop;
+         }
 
        if(IsFakeNode(node1) || IsFakeNode(node2))
          {
@@ -390,7 +398,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
        results->prev_segment=NO_SEGMENT;
     else
       {
-       index_t superseg=FindSuperSegment(nodes,segments,ways,relations,begin->start_node,begin->prev_segment);
+       index_t superseg=FindSuperSegment(nodes,segments,ways,relations,profile,begin->start_node,begin->prev_segment);
 
        results->prev_segment=superseg;
       }
@@ -406,15 +414,17 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
  while(result3)
    {
     if((results->start_node!=result3->node || results->prev_segment!=result3->segment) &&
-       !IsFakeNode(result3->node) && IsSuperNode(LookupNode(nodes,result3->node,5)))
+       !IsFakeNode(result3->node) && IsSuperNode(LookupNode(nodes,result3->node,3)))
       {
        Result *result5=result1;
-       index_t superseg=FindSuperSegment(nodes,segments,ways,relations,result3->node,result3->segment);
+       index_t superseg=FindSuperSegment(nodes,segments,ways,relations,profile,result3->node,result3->segment);
 
        if(superseg!=result3->segment)
          {
           result5=InsertResult(results,result3->node,result3->segment);
 
+          result5->score=result3->score;
+
           result5->prev=result1;
          }
 
@@ -494,7 +504,15 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
        /* must obey one-way restrictions (unless profile allows) */
        if(profile->oneway && IsOnewayTo(segmentp,node1))
-          goto endloop;
+         {
+          if(profile->allow!=Transports_Bicycle)
+             goto endloop;
+
+          wayp=LookupWay(ways,segmentp->way,1);
+
+          if(!(wayp->type&Highway_CycleBothWays))
+             goto endloop;
+         }
 
        seg2=IndexSegment(segments,segmentp); /* segment cannot be a fake segment (must be a super-segment) */
 
@@ -682,12 +700,14 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
   Relations *relations The set of relations to use.
 
+  Profile *profile The profile containing the transport type, speeds and allowed highways.
+
   index_t finish_node The super-node that the route ends at.
 
   index_t finish_segment The segment that the route ends with.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,index_t finish_node,index_t finish_segment)
+static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t finish_node,index_t finish_segment)
 {
  Node *supernodep;
  Segment *supersegmentp;
@@ -695,15 +715,15 @@ static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relat
  if(IsFakeSegment(finish_segment))
     finish_segment=IndexRealSegment(finish_segment);
 
- supernodep=LookupNode(nodes,finish_node,5); /* finish_node cannot be a fake node (must be a super-node) */
- supersegmentp=LookupSegment(segments,finish_segment,2); /* finish_segment cannot be a fake segment. */
+ supernodep=LookupNode(nodes,finish_node,3); /* finish_node cannot be a fake node (must be a super-node) */
+ supersegmentp=LookupSegment(segments,finish_segment,4); /* finish_segment cannot be a fake segment. */
 
  if(IsSuperSegment(supersegmentp))
     return(finish_segment);
 
  /* Loop across all segments */
 
- supersegmentp=FirstSegment(segments,supernodep,3); /* supernode cannot be a fake node (must be a super-node) */
+ supersegmentp=FirstSegment(segments,supernodep,4); /* supernode cannot be a fake node (must be a super-node) */
 
  while(supersegmentp)
    {
@@ -715,7 +735,7 @@ static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relat
 
        start_node=OtherNode(supersegmentp,finish_node);
 
-       results=FindSuperRoute(nodes,segments,ways,relations,start_node,finish_node);
+       results=FindSuperRoute(nodes,segments,ways,relations,profile,start_node,finish_node);
 
        if(!results)
           continue;
@@ -753,12 +773,14 @@ static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relat
 
   Relations *relations The set of relations to use.
 
+  Profile *profile The profile containing the transport type, speeds and allowed highways.
+
   index_t start_node The start node.
 
   index_t finish_node The finish node.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,index_t start_node,index_t finish_node)
+static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t start_node,index_t finish_node)
 {
  Results *results;
  Queue   *queue;
@@ -791,11 +813,11 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
     node1=result1->node;
     seg1=result1->segment;
 
-    node1p=LookupNode(nodes,node1,1); /* node1 cannot be a fake node */
+    node1p=LookupNode(nodes,node1,3); /* node1 cannot be a fake node */
 
     /* Loop across all segments */
 
-    segmentp=FirstSegment(segments,node1p,1); /* node1 cannot be a fake node */
+    segmentp=FirstSegment(segments,node1p,3); /* node1 cannot be a fake node */
 
     while(segmentp)
       {
@@ -809,7 +831,17 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
 
        /* must obey one-way restrictions */
        if(IsOnewayTo(segmentp,node1))
-          goto endloop;
+         {
+          Way *wayp;
+
+          if(profile->allow!=Transports_Bicycle)
+             goto endloop;
+
+          wayp=LookupWay(ways,segmentp->way,2);
+
+          if(!(wayp->type&Highway_CycleBothWays))
+             goto endloop;
+         }
 
        seg2=IndexSegment(segments,segmentp);
 
@@ -819,7 +851,7 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
 
        node2=OtherNode(segmentp,node1);
 
-       node2p=LookupNode(nodes,node2,2); /* node2 cannot be a fake node */
+       node2p=LookupNode(nodes,node2,4); /* node2 cannot be a fake node */
 
        /* must not pass over super-node */
        if(node2!=finish_node && IsSuperNode(node2p))
@@ -973,7 +1005,15 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
        /* must obey one-way restrictions (unless profile allows) */
        if(profile->oneway && IsOnewayTo(segmentp,node1))
-          goto endloop;
+         {
+          if(profile->allow!=Transports_Bicycle)
+             goto endloop;
+
+          wayp=LookupWay(ways,segmentp->way,1);
+
+          if(!(wayp->type&Highway_CycleBothWays))
+             goto endloop;
+         }
 
        if(IsFakeNode(node1) || IsFakeNode(node2))
          {
@@ -1211,7 +1251,7 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
          }
 
     if(!IsFakeNode(result3->node))
-       if(IsSuperNode(LookupNode(nodes,result3->node,5)))
+       if(IsSuperNode(LookupNode(nodes,result3->node,3)))
           InsertInQueue(queue,result3,result3->score);
 
     result3=NextResult(results,result3);
@@ -1268,7 +1308,15 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
 
        /* must obey one-way restrictions (unless profile allows) */
        if(profile->oneway && IsOnewayTo(segmentp,node1))
-          goto endloop;
+         {
+          if(profile->allow!=Transports_Bicycle)
+             goto endloop;
+
+          wayp=LookupWay(ways,segmentp->way,1);
+
+          if(!(wayp->type&Highway_CycleBothWays))
+             goto endloop;
+         }
 
        if(IsFakeNode(node1) || IsFakeNode(node2))
          {
@@ -1492,7 +1540,15 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
 
        /* must obey one-way restrictions (unless profile allows) */
        if(profile->oneway && IsOnewayFrom(segmentp,node1)) /* working backwards => disallow oneway *from* node1 */
-          goto endloop;
+         {
+          if(profile->allow!=Transports_Bicycle)
+             goto endloop;
+
+          wayp=LookupWay(ways,segmentp->way,1);
+
+          if(!(wayp->type&Highway_CycleBothWays))
+             goto endloop;
+         }
 
        node2=OtherNode(segmentp,node1);
 
@@ -1508,8 +1564,21 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
          }
 
        /* must not perform U-turn (unless profile allows) */
-       if(profile->turns && (seg1==seg2 || seg1==seg2r || seg1r==seg2 || (seg1r==seg2r && IsFakeUTurn(seg1,seg2))))
-          goto endloop;
+       if(profile->turns)
+         {
+          if(IsFakeNode(node1) || !IsSuperNode(node1p))
+            {
+             if(seg1==seg2 || seg1==seg2r || seg1r==seg2 || (seg1r==seg2r && IsFakeUTurn(seg1,seg2)))
+                goto endloop;
+            }
+          else
+            {
+             index_t superseg=FindSuperSegment(nodes,segments,ways,relations,profile,node1,seg1);
+
+             if(seg2==superseg)
+                goto endloop;
+            }
+         }
 
        /* must obey turn relations */
        if(turnrelation!=NO_RELATION)
