@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -40,6 +40,8 @@
 #include "xmlparse.h"
 
 
+#define DEBUG 0
+
 /* Constants */
 
 #define IMP_IGNORE      -1      /*+ Ignore this point. +*/
@@ -61,7 +63,7 @@
 extern int option_quickest;
 
 /*+ The options to select the format of the output. +*/
-extern int option_html,option_gpx_track,option_gpx_route,option_text,option_text_all;
+extern int option_html,option_gpx_track,option_gpx_route,option_text,option_text_all,option_stdout;
 
 
 /* Local variables */
@@ -88,7 +90,7 @@ static char junction_other_way[Highway_Count][Highway_Count]=
 /*++++++++++++++++++++++++++++++++++++++
   Print the optimum route between two nodes.
 
-  Results **results The set of results to print (some may be NULL - ignore them).
+  Results **results The set of results to print (consecutive in array even if not consecutive waypoints).
 
   int nresults The number of results in the list.
 
@@ -106,87 +108,104 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
  FILE *htmlfile=NULL,*gpxtrackfile=NULL,*gpxroutefile=NULL,*textfile=NULL,*textallfile=NULL;
 
  char *prev_bearing=NULL,*prev_wayname=NULL;
+ index_t prev_node=NO_NODE;
  distance_t cum_distance=0;
  duration_t cum_duration=0;
 
- int point=1;
+ int point=0;
  int segment_count=0,route_count=0;
  int point_count=0;
  int roundabout=0;
 
  /* Open the files */
 
- if(option_quickest==0)
+ if(option_stdout)
    {
-    /* Print the result for the shortest route */
-
     if(option_html)
-       htmlfile    =fopen("shortest.html","w");
+       htmlfile    =stdout;
     if(option_gpx_track)
-       gpxtrackfile=fopen("shortest-track.gpx","w");
+       gpxtrackfile=stdout;
     if(option_gpx_route)
-       gpxroutefile=fopen("shortest-route.gpx","w");
+       gpxroutefile=stdout;
     if(option_text)
-       textfile    =fopen("shortest.txt","w");
+       textfile    =stdout;
     if(option_text_all)
-       textallfile =fopen("shortest-all.txt","w");
-
-    if(option_html && !htmlfile)
-       fprintf(stderr,"Warning: Cannot open file 'shortest.html' for writing [%s].\n",strerror(errno));
-    if(option_gpx_track && !gpxtrackfile)
-       fprintf(stderr,"Warning: Cannot open file 'shortest-track.gpx' for writing [%s].\n",strerror(errno));
-    if(option_gpx_route && !gpxroutefile)
-       fprintf(stderr,"Warning: Cannot open file 'shortest-route.gpx' for writing [%s].\n",strerror(errno));
-    if(option_text && !textfile)
-       fprintf(stderr,"Warning: Cannot open file 'shortest.txt' for writing [%s].\n",strerror(errno));
-    if(option_text_all && !textallfile)
-       fprintf(stderr,"Warning: Cannot open file 'shortest-all.txt' for writing [%s].\n",strerror(errno));
+       textallfile =stdout;
    }
  else
    {
-    /* Print the result for the quickest route */
+    if(option_quickest==0)
+      {
+       /* Print the result for the shortest route */
 
-    if(option_html)
-       htmlfile    =fopen("quickest.html","w");
-    if(option_gpx_track)
-       gpxtrackfile=fopen("quickest-track.gpx","w");
-    if(option_gpx_route)
-       gpxroutefile=fopen("quickest-route.gpx","w");
-    if(option_text)
-       textfile    =fopen("quickest.txt","w");
-    if(option_text_all)
-       textallfile =fopen("quickest-all.txt","w");
+       if(option_html)
+          htmlfile    =fopen("shortest.html","w");
+       if(option_gpx_track)
+          gpxtrackfile=fopen("shortest-track.gpx","w");
+       if(option_gpx_route)
+          gpxroutefile=fopen("shortest-route.gpx","w");
+       if(option_text)
+          textfile    =fopen("shortest.txt","w");
+       if(option_text_all)
+          textallfile =fopen("shortest-all.txt","w");
 
-    if(option_html && !htmlfile)
-       fprintf(stderr,"Warning: Cannot open file 'quickest.html' for writing [%s].\n",strerror(errno));
-    if(option_gpx_track && !gpxtrackfile)
-       fprintf(stderr,"Warning: Cannot open file 'quickest-track.gpx' for writing [%s].\n",strerror(errno));
-    if(option_gpx_route && !gpxroutefile)
-       fprintf(stderr,"Warning: Cannot open file 'quickest-route.gpx' for writing [%s].\n",strerror(errno));
-    if(option_text && !textfile)
-       fprintf(stderr,"Warning: Cannot open file 'quickest.txt' for writing [%s].\n",strerror(errno));
-    if(option_text_all && !textallfile)
-       fprintf(stderr,"Warning: Cannot open file 'quickest-all.txt' for writing [%s].\n",strerror(errno));
+       if(option_html && !htmlfile)
+          fprintf(stderr,"Warning: Cannot open file 'shortest.html' for writing [%s].\n",strerror(errno));
+       if(option_gpx_track && !gpxtrackfile)
+          fprintf(stderr,"Warning: Cannot open file 'shortest-track.gpx' for writing [%s].\n",strerror(errno));
+       if(option_gpx_route && !gpxroutefile)
+          fprintf(stderr,"Warning: Cannot open file 'shortest-route.gpx' for writing [%s].\n",strerror(errno));
+       if(option_text && !textfile)
+          fprintf(stderr,"Warning: Cannot open file 'shortest.txt' for writing [%s].\n",strerror(errno));
+       if(option_text_all && !textallfile)
+          fprintf(stderr,"Warning: Cannot open file 'shortest-all.txt' for writing [%s].\n",strerror(errno));
+      }
+    else
+      {
+       /* Print the result for the quickest route */
+
+       if(option_html)
+          htmlfile    =fopen("quickest.html","w");
+       if(option_gpx_track)
+          gpxtrackfile=fopen("quickest-track.gpx","w");
+       if(option_gpx_route)
+          gpxroutefile=fopen("quickest-route.gpx","w");
+       if(option_text)
+          textfile    =fopen("quickest.txt","w");
+       if(option_text_all)
+          textallfile =fopen("quickest-all.txt","w");
+
+       if(option_html && !htmlfile)
+          fprintf(stderr,"Warning: Cannot open file 'quickest.html' for writing [%s].\n",strerror(errno));
+       if(option_gpx_track && !gpxtrackfile)
+          fprintf(stderr,"Warning: Cannot open file 'quickest-track.gpx' for writing [%s].\n",strerror(errno));
+       if(option_gpx_route && !gpxroutefile)
+          fprintf(stderr,"Warning: Cannot open file 'quickest-route.gpx' for writing [%s].\n",strerror(errno));
+       if(option_text && !textfile)
+          fprintf(stderr,"Warning: Cannot open file 'quickest.txt' for writing [%s].\n",strerror(errno));
+       if(option_text_all && !textallfile)
+          fprintf(stderr,"Warning: Cannot open file 'quickest-all.txt' for writing [%s].\n",strerror(errno));
+      }
    }
 
  /* Print the head of the files */
 
  if(htmlfile)
    {
-    fprintf(htmlfile,"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n");
-    fprintf(htmlfile,"<HTML>\n");
+    fprintf(htmlfile,"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n");
+    fprintf(htmlfile,"<html>\n");
     if(translate_xml_copyright_creator[0] && translate_xml_copyright_creator[1])
        fprintf(htmlfile,"<!-- %s : %s -->\n",translate_xml_copyright_creator[0],translate_xml_copyright_creator[1]);
     if(translate_xml_copyright_source[0] && translate_xml_copyright_source[1])
        fprintf(htmlfile,"<!-- %s : %s -->\n",translate_xml_copyright_source[0],translate_xml_copyright_source[1]);
     if(translate_xml_copyright_license[0] && translate_xml_copyright_license[1])
        fprintf(htmlfile,"<!-- %s : %s -->\n",translate_xml_copyright_license[0],translate_xml_copyright_license[1]);
-    fprintf(htmlfile,"<HEAD>\n");
-    fprintf(htmlfile,"<TITLE>");
+    fprintf(htmlfile,"<head>\n");
+    fprintf(htmlfile,"<title>");
     fprintf(htmlfile,translate_html_title,option_quickest?translate_xml_route_quickest:translate_xml_route_shortest);
-    fprintf(htmlfile,"</TITLE>\n");
-    fprintf(htmlfile,"<META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
-    fprintf(htmlfile,"<STYLE type=\"text/css\">\n");
+    fprintf(htmlfile,"</title>\n");
+    fprintf(htmlfile,"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
+    fprintf(htmlfile,"<style type=\"text/css\">\n");
     fprintf(htmlfile,"<!--\n");
     fprintf(htmlfile,"   table   {table-layout: fixed; border: none; border-collapse: collapse;}\n");
     fprintf(htmlfile,"   table.c {color: grey; font-size: x-small;} /* copyright */\n");
@@ -204,12 +223,12 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
     fprintf(htmlfile,"   span.t  {font-variant: small-caps;} /* turn */\n");
     fprintf(htmlfile,"   span.b  {font-variant: small-caps;} /* bearing */\n");
     fprintf(htmlfile,"-->\n");
-    fprintf(htmlfile,"</STYLE>\n");
-    fprintf(htmlfile,"</HEAD>\n");
-    fprintf(htmlfile,"<BODY>\n");
-    fprintf(htmlfile,"<H1>");
+    fprintf(htmlfile,"</style>\n");
+    fprintf(htmlfile,"</head>\n");
+    fprintf(htmlfile,"<body>\n");
+    fprintf(htmlfile,"<h1>");
     fprintf(htmlfile,translate_html_title,option_quickest?translate_xml_route_quickest:translate_xml_route_shortest);
-    fprintf(htmlfile,"</H1>\n");
+    fprintf(htmlfile,"</h1>\n");
     fprintf(htmlfile,"<table>\n");
    }
 
@@ -305,16 +324,29 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
  /* Loop through all the sections of the route and print them */
 
- while(!results[point])
-    point++;
-
- while(point<=nresults)
+ do
    {
+    int first=1;
     int next_point=point;
     distance_t junc_distance=0;
     duration_t junc_duration=0;
     Result *result;
 
+#if DEBUG
+    printf("Route section %d - waypoint %d to waypoint %d\n",point,results[point]->start_waypoint,results[point]->finish_waypoint);
+    printf("  start_node=%"Pindex_t" prev_segment=%"Pindex_t"\n",results[point]->start_node,results[point]->prev_segment);
+    printf("  finish_node=%"Pindex_t" last_segment=%"Pindex_t"\n",results[point]->finish_node,results[point]->last_segment);
+
+    Result *r=FindResult(results[point],results[point]->start_node,results[point]->prev_segment);
+
+    while(r)
+      {
+       printf("    node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+       r=r->next;
+      }
+#endif
+
     result=FindResult(results[point],results[point]->start_node,results[point]->prev_segment);
 
     /* Print the start of the segment */
@@ -357,31 +389,33 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
        next_result=result->next;
 
        if(!next_result)
-          for(next_point=point+1;next_point<=nresults;next_point++)
-             if(results[next_point])
-               {
-                next_result=FindResult(results[next_point],results[next_point]->start_node,results[next_point]->prev_segment);
-                next_result=next_result->next;
-                break;
-               }
+         {
+          next_point++;
+
+          if(next_point<nresults)
+            {
+             next_result=FindResult(results[next_point],results[next_point]->start_node,results[next_point]->prev_segment);
+             next_result=next_result->next;
+            }
+         }
 
        /* Calculate the information about this segment */
 
-       if(result->node!=results[point]->start_node) /* not first point of a section of the route */
+       if(IsFakeSegment(result->segment))
          {
-          if(IsFakeSegment(result->segment))
-            {
-             resultsegmentp=LookupFakeSegment(result->segment);
-             realsegment=IndexRealSegment(result->segment);
-            }
-          else
-            {
-             resultsegmentp=LookupSegment(segments,result->segment,2);
-             realsegment=result->segment;
-            }
+          resultsegmentp=LookupFakeSegment(result->segment);
+          realsegment=IndexRealSegment(result->segment);
+         }
+       else
+         {
+          resultsegmentp=LookupSegment(segments,result->segment,2);
+          realsegment=result->segment;
+         }
 
-          resultwayp=LookupWay(ways,resultsegmentp->way,1);
+       resultwayp=LookupWay(ways,resultsegmentp->way,1);
 
+       if(!first)               /* not first point of a section of the route */
+         {
           seg_distance+=DISTANCE(resultsegmentp->distance);
           seg_duration+=Duration(resultsegmentp,resultwayp,profile);
 
@@ -424,26 +458,57 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
                }
              else
                {
-                Segment *segmentp=FirstSegment(segments,resultnodep,3);
+                Segment *segmentp;
+
+                if(resultnodep)
+                   segmentp=FirstSegment(segments,resultnodep,3);
+                else
+                   segmentp=FirstFakeSegment(result->node);
 
                 do
                   {
                    index_t othernode=OtherNode(segmentp,result->node);
+                   index_t thissegment;
 
-                   if(othernode!=result->prev->node && IndexSegment(segments,segmentp)!=realsegment)
-                      if(IsNormalSegment(segmentp) && (!profile->oneway || !IsOnewayTo(segmentp,result->node)))
+                   if(IsFakeNode(result->node))
+                      thissegment=IndexFakeSegment(segmentp);
+                   else
+                      thissegment=IndexSegment(segments,segmentp);
+
+                   if(othernode!=prev_node && othernode!=next_result->node &&
+                      thissegment!=realsegment && IsNormalSegment(segmentp))
+                     {
+                      int canexit=1;
+
+                      if(profile->oneway && IsOnewayTo(segmentp,result->node))
+                        {
+                         if(profile->allow!=Transports_Bicycle)
+                            canexit=0;
+                         else
+                           {
+                            Way *wayp=LookupWay(ways,segmentp->way,3);
+
+                            if(!(wayp->type&Highway_CycleBothWays))
+                               canexit=0;
+                           }
+                        }
+
+                      if(canexit)
                         {
                          Way *wayp=LookupWay(ways,segmentp->way,3);
 
                          if(!(wayp->type&Highway_Roundabout))
-                            if(othernode!=next_result->node)
-                              {
-                               roundabout++;
-                               important=IMP_RB_NOT_EXIT;
-                              }
+                           {
+                            roundabout++;
+                            important=IMP_RB_NOT_EXIT;
+                           }
                         }
+                     }
 
-                   segmentp=NextSegment(segments,segmentp,result->node);
+                   if(resultnodep)
+                      segmentp=NextSegment(segments,segmentp,result->node);
+                   else
+                      segmentp=NextFakeSegment(segmentp,result->node);
                   }
                 while(segmentp);
                }
@@ -458,14 +523,14 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
        /* Decide if this is an important junction */
 
-       if(roundabout)           /* roundabout */
-          ;
-       else if(point_count==0)  /* first point overall = Waypoint */
+       if(point_count==0)  /* first point overall = Waypoint */
           important=IMP_WAYPOINT;
-       else if(result->node==results[point]->finish_node) /* Waypoint */
+       else if(result->next==NULL) /* Waypoint */
           important=IMP_WAYPOINT;
-       else if(result->node==results[point]->start_node) /* first point of a section of the route */
+       else if(first)           /* first point of a section of the route */
           important=IMP_IGNORE;
+       else if(roundabout)      /* roundabout */
+          ;
        else if(realsegment==next_realsegment) /* U-turn */
           important=IMP_UTURN;
        else if(resultnodep && (resultnodep->flags&NODE_MINIRNDBT))
@@ -478,8 +543,24 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
             {
              index_t seg=IndexSegment(segments,segmentp);
 
-             if(seg!=realsegment)
-                if(IsNormalSegment(segmentp) && (!profile->oneway || !IsOnewayTo(segmentp,result->node)))
+             if(seg!=realsegment && IsNormalSegment(segmentp))
+               {
+                int cango=1;
+
+                if(profile->oneway && IsOnewayTo(segmentp,result->node))
+                  {
+                   if(profile->allow!=Transports_Bicycle)
+                      cango=0;
+                   else
+                     {
+                      Way *wayp=LookupWay(ways,segmentp->way,3);
+
+                      if(!(wayp->type&Highway_CycleBothWays))
+                         cango=0;
+                     }
+                  }
+
+                if(cango)
                   {
                    Way *wayp=LookupWay(ways,segmentp->way,3);
 
@@ -499,6 +580,7 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
                          important=IMP_JUNCT_CONT;
                      }
                   }
+               }
 
              segmentp=NextSegment(segments,segmentp,result->node);
             }
@@ -507,7 +589,7 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
        /* Calculate the strings to be used */
 
-       if(resultwayp && textallfile)
+       if(!first && textallfile)
          {
           waynameraw=WayName(ways,resultwayp);
           if(!*waynameraw)
@@ -520,9 +602,13 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
        if(next_result && important>IMP_JUNCT_CONT)
          {
-          if(resultsegmentp && (htmlfile || textfile))
+          if(!first && (htmlfile || textfile))
             {
-             turn_int=(int)TurnAngle(nodes,resultsegmentp,next_resultsegmentp,result->node);
+             if(DISTANCE(resultsegmentp->distance)==0 || DISTANCE(next_resultsegmentp->distance)==0)
+                turn_int=0;
+             else
+                turn_int=(int)TurnAngle(nodes,resultsegmentp,next_resultsegmentp,result->node);
+
              turn=translate_xml_turn[((202+turn_int)/45)%8];
             }
 
@@ -537,7 +623,11 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
           if(htmlfile || gpxroutefile || textfile)
             {
-             next_bearing_int=(int)BearingAngle(nodes,next_resultsegmentp,next_result->node);
+             if(DISTANCE(next_resultsegmentp->distance)==0)
+                next_bearing_int=(int)BearingAngle(nodes,resultsegmentp,result->node);
+             else
+                next_bearing_int=(int)BearingAngle(nodes,next_resultsegmentp,next_result->node);
+
              next_bearing=translate_xml_heading[(4+(22+next_bearing_int)/45)%8];
             }
          }
@@ -586,7 +676,7 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
                }
              else if(next_result) /* middle point */
                {
-                if(roundabout>1)
+                if(roundabout>1 && important!=IMP_WAYPOINT)
                   {
                    /* <tr class='n'><td class='l'>At:<td class='r'>Roundabout, take <span class='t'>the *Nth* exit</span> heading <span class='b'>*heading*</span> */
                    fprintf(htmlfile,"<tr class='n'><td class='l'>%s:<td class='r'>",translate_html_rbnode[0]);
@@ -784,6 +874,8 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
        if(important>IMP_JUNCT_CONT)
           point_count++;
+
+       first=0;
       }
     while(point==next_point);
 
@@ -793,7 +885,13 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
        fprintf(gpxtrackfile,"</trkseg>\n");
 
     point=next_point;
+
+    if(result)
+       prev_node=result->node;
+    else
+       prev_node=NO_NODE;
    }
+ while(point<nresults);
 
  /* Print the tail of the files */
 
@@ -816,8 +914,8 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
        fprintf(htmlfile,"</table>\n");
       }
 
-    fprintf(htmlfile,"</BODY>\n");
-    fprintf(htmlfile,"</HTML>\n");
+    fprintf(htmlfile,"</body>\n");
+    fprintf(htmlfile,"</html>\n");
    }
 
  if(gpxtrackfile)
@@ -834,14 +932,17 @@ void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,W
 
  /* Close the files */
 
- if(htmlfile)
-    fclose(htmlfile);
- if(gpxtrackfile)
-    fclose(gpxtrackfile);
- if(gpxroutefile)
-    fclose(gpxroutefile);
- if(textfile)
-    fclose(textfile);
- if(textallfile)
-    fclose(textallfile);
+ if(!option_stdout)
+   {
+    if(htmlfile)
+       fclose(htmlfile);
+    if(gpxtrackfile)
+       fclose(gpxtrackfile);
+    if(gpxroutefile)
+       fclose(gpxroutefile);
+    if(textfile)
+       fclose(textfile);
+    if(textallfile)
+       fclose(textallfile);
+   }
 }
